from .client import RightClient, login_using_env_and_get_client
import json
import os
from . import dir_config as dc

from .utils import *


def get_waiting_items():
    """
    returns list of (commit_hash, record_id)
    """
    items = []
    for name in os.listdir(dc.wait_dir):
        commit_hash = name

        json_info = read_json_file(os.path.join(dc.wait_dir, name))
        token = json_info['token']

        items.append((commit_hash, token))

    return items


def is_packaged(record: dict):
    return record['transactionId'] is not None


def do_fetch():

    client = RightClient()

    for commit_hash, token in get_waiting_items():

        record = client.outofbox_get_record(token)

        if is_packaged(record):
            write_json_to_file(record, path=os.path.join(
                dc.packaged_dir, commit_hash))
            os.remove(os.path.join(dc.wait_dir, commit_hash))
            print("packaged:", commit_hash)
        else:
            print("still not packaged:", commit_hash)
            # print(record)



import hashlib
import json


def read_json_file(filename):
    return json.load(open(filename, 'r', encoding='utf8'))


def write_json_to_file(obj, path):
    with open(path, 'w', encoding='utf8') as f:
        json.dump(obj, f, ensure_ascii=False, indent=4)


def file_sha256(filename):
    with open(filename, 'rb') as f:
        h = hashlib.sha256()
        h.update(f.read())
        return h.hexdigest().lower()



from . import dir_config
import pathspec
import os


def list_all_files_in_workdir():
    all_files = []

    for dirpath, dirs, files in os.walk('.'):
        for name in files:
            filename = os.path.join(dirpath, name)
            filename = filename.replace("\\", "/")
            all_files.append(filename)

    return all_files


def list_included_file_in_workdir(exclude_spec_lines: "list[str]"):
    spec = pathspec.PathSpec.from_lines(
        pathspec.patterns.GitWildMatchPattern, exclude_spec_lines)

    all_files = list_all_files_in_workdir()
    included = []

    for file in all_files:
        if not spec.match_file(file):
            included.append(file)

    return included


def list_files_with_gitignore_and_rightignore():
    exclude_spec_lines = []
    exclude_spec_lines.append(".git")
    exclude_spec_lines.append(dir_config.right_dir)

    for ignorefile in ['.gitignore', '.rightignore']:
        if os.path.exists(ignorefile):
            print(f"using ignore file: {ignorefile}")

            with open(ignorefile, 'r', encoding='utf8') as f:
                lines = f.readlines()

            exclude_spec_lines += lines

    files = list_included_file_in_workdir(exclude_spec_lines)
    return files


# files = list_files_with_gitignore_and_rightignore()
# print(len(files))

# for file in files:
#     print(file)



import pathlib
import os

right_dir = 'copyrightstore'

wait_dir = f"{right_dir}/waiting"
packaged_dir = f"{right_dir}/packaged"
index_file = f'{right_dir}/index.json'


def create_dir(path):
    fileinfo = pathlib.Path(path)
    if fileinfo.exists():
        assert fileinfo.is_dir()
    else:
        fileinfo.mkdir()


def ensure_all_dir_created():
    create_dir(right_dir)
    create_dir(wait_dir)
    create_dir(packaged_dir)



import os
import requests


class RightClient:
    def __init__(self, base_url="https://rightchain.cc") -> None:
        if os.environ.get('RIGHT_BASEURL') is not None:
            base_url = os.environ.get('RIGHT_BASEURL')
            print(f"using RIGHT_BASEURL: {base_url}")

        self.base_url = base_url
        self.sess = requests.session()

    def login(self, username, password):
        res = self.sess.post(self.base_url+"/api/access/login", json={
            "username": username,
            "password": password,
        })

        if not res.ok:
            raise Exception(f"登录失败. 响应内容：{res.text}")

    def create_record(self, name, text):
        res = self.sess.post(self.base_url + "/api/records", json={
            "name": name, "text": text
        })

        assert res.ok
        return res.json()['id']

    def get_record(self, id):
        res = self.sess.get(self.base_url + f"/api/records/{id}")
        assert res.ok, res.text
        return res.json()

    def outofbox_create_record(self, hashstr):
        assert len(hashstr) <= 64

        res = self.sess.post(self.base_url+'/api/out-of-box/create-record', json={
            'hash':  hashstr
        })

        assert res.ok
        return res.json()['token']

    def outofbox_get_record(self, token):
        res = self.sess.get(self.base_url+'/api/out-of-box/get-record', params={
            'token': token
        })

        assert res.ok

        return res.json()


def login_using_env_and_get_client():
    try:
        username = os.environ["RIGHT_USERNAME"]
        password = os.environ["RIGHT_PASSWORD"]
    except Exception:
        print(f"请设置环境变量：RIGHT_USERNAME, RIGHT_PASSWORD")
        raise

    client = RightClient()

    client.login(username, password)

    return client



from . import file_lister
import pathlib
from . import dir_config as dc
from .client import RightClient, login_using_env_and_get_client
import json
import os
import hashlib
import subprocess
from .utils import *


def get_previous_commit_hash():
    cmd = 'git log --pretty=format:%H -1'
    outp = subprocess.check_output(cmd)
    return outp.decode('utf8')


def unix_style(s: str):
    """ change \\ to / """
    return s.replace("\\", '/')


def create_empty_file(path):
    pathlib.Path(path).touch()


def get_sha256_of_files(files: "list[str]"):
    """
    returns list of {name: ..., hash: ...}
    """
    data = []

    for file in files:
        hash = file_sha256(file)
        data.append({
            "name": file,
            "hash": hash,
        })

    return data


def try_read_copyright_info():
    copyright_info = "copyright-info"
    if os.path.exists(copyright_info):
        print(f'using copyright info: {copyright_info}')

        with open(copyright_info, 'r', encoding='utf8') as f:
            info = f.read()
            return info
    else:
        return None


def create_index():
    files = file_lister.list_files_with_gitignore_and_rightignore()
    data = get_sha256_of_files(files)

    data = [p['hash'] for p in data] # only hash for filename privacy

    # write to index file

    prev_commit = get_previous_commit_hash()  # return "" when no prev commit

    index_content = {
        'previousCommit': prev_commit,
        'copyrightInfo': try_read_copyright_info(),
        "data": data
    }

    # save index content to index file
    write_json_to_file(index_content, dc.index_file)

    print(f"Index created: {dc.index_file}")


def push_index():
    # post and get record id

    client = RightClient()

    hash_of_index_file = file_sha256(dc.index_file)

    prev_commit = read_json_file(dc.index_file)['previousCommit']

    token = client.outofbox_create_record(hash_of_index_file)

    wait_record_file = os.path.join(dc.wait_dir, prev_commit)

    write_json_to_file({"token": token}, wait_record_file)

    print(f"Pushed and create waiting file: {wait_record_file}")



import setuptools

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

setuptools.setup(
    name="rightchain",
    version="0.0.6",
    author="Example Author",
    author_email="author@example.com",
    description="A small example package",
    # long_description=long_description,
    # long_description_content_type="text/markdown",
    # url="https://github.com/pypa/sampleproject",
    project_urls={
        # "Bug Tracker": "https://github.com/pypa/sampleproject/issues",
    },
    classifiers=[
        # "Programming Language :: Python :: 3",
        # "License :: OSI Approved :: MIT License",
        # "Operating System :: OS Independent",
    ],
    package_dir={"": "src"},
    packages=setuptools.find_packages(where="src"),
    python_requires=">=3.7",
    scripts=["src/rightchain/rccc", "src/rightchain/rccc.ps1"],
    install_requires=[
        "requests",
        "pathspec"
    ]
)



import os
from . import dir_config
from .dir_config import ensure_all_dir_created
from .fetch_record import do_fetch
from .create_index import create_index, push_index
from . import client as api_client
import sys
import argparse

if __name__ == "__main__":

    parser = argparse.ArgumentParser(
        prog="rccc",
        description='Command line tool for copyright on blockchain')

    # parser.add_argument('command', choices=[
    #                     'create-index', 'push-index', 'index', 'fetch', 'test-login'], help="action to perform")

    parser.add_argument('command', choices=[
        'create-index', 'push-index', 'index', 'fetch'], help="action to perform")

    args = parser.parse_args()

    command = args.command

    # in case git not remembering empty folders
    ensure_all_dir_created()

    # if command == 'test-login':
    #     api_client.login_using_env_and_get_client()
    #     print("Login test success!")
    #     exit(0)

    if command == 'create-index':
        create_index()
        print("Create index complete.")
        exit(0)

    if command == 'push-index':
        push_index()
        # print("Pushed index to server.")
        exit(0)

    if command == 'index':
        create_index()
        push_index()
        exit(0)

    if command == 'fetch':
        do_fetch()
        print("Fetch from server complete")
        exit(0)

    raise Exception("Not reached")
